// ComLog.cpp : Implementation of CComLog
#include "stdafx.h"
#include "LogBook.h"
#include "ComLog.h"
#include "ComLogbook.h"
 
/////////////////////////////////////////////////////////////////////////////
// CComLog

CComLog::CComLog()
{
}

CComLog::~CComLog()
{
}

HRESULT  CComLog::FinalConstruct(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   return S_OK;
}

void CComLog::FinalRelease(void)
{
   AFX_MANAGE_STATE(AfxGetStaticModuleState())

   if (m_LogFile.m_pStream)
   {
      TRY
      {
         m_LogFile.Flush();
         m_LogFile.Close();   
      }
      CATCH(CFileException, e)
      {
         ATLASSERT(FALSE);
      }
      END_CATCH
   }
}

/////////////////////////////////////////////////////////////////////////////
// IObjectConstruct
STDMETHODIMP CComLog::Construct(IDispatch * pCtorObj)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())
   HRESULT hres = S_OK;
	IObjectConstructString* pString = NULL;
   hres = pCtorObj->QueryInterface(IID_IObjectConstructString, (void **)&pString);
   ASSERT_AND_RETURN(hres);

	BSTR	bstrConstruct;
	hres = pString->get_ConstructString(&bstrConstruct);
   pString->Release();
   ASSERT_AND_RETURN(hres);

   CString sPath = bstrConstruct;
   SysFreeString(bstrConstruct);

   BOOL bres = TRUE;

   if (sPath.IsEmpty())
   {

      CString sLogName;
      bres = sLogName.LoadString(IDS_LOG_FILENAME);
      if (FALSE == bres)
      {
         sLogName = "Logbook.htm";
      }

      TCHAR lpFileName[MAX_PATH];
      DWORD dwLength = ::GetModuleFileName(AfxGetInstanceHandle(), lpFileName, MAX_PATH - 2);
      if (dwLength)
      {
         sPath = lpFileName;
         CString sFileName = CEventLogger::FindPathInFileName(lpFileName);
         int nFileNameIndex = sPath.Find(sFileName);
         sPath.Delete(nFileNameIndex, sFileName.GetLength());
      }

      sPath += sLogName;
   }

   CFileException fileError;
   bres = m_LogFile.Open(sPath, 
                         CFile::shareDenyNone | CFile::modeCreate |
                         CFile::modeWrite     | CFile::typeText, &fileError); 
   if (FALSE == bres)
   {
      return E_FAIL;
   }

   TRY
   {
      m_LogFile.Seek(0, CFile::end);
   }
   CATCH(CFileException, e)
   {
      return E_FAIL;
   }
   END_CATCH

   return SetTableHeader();
}
////////////////////////////////////////////////////////////
//ILogbook implementation

STDMETHODIMP CComLog::LogError(LOG_ENTRY *pErrorEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;
   CString sTableEnd;
   hres = NewRow(pErrorEntry, sTableEnd);
   ASSERT_AND_RETURN(hres);

   CString sStr;
   sStr = pErrorEntry->bstrMethodName;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   sStr = pErrorEntry->bstrDescription;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   _com_error comErr(pErrorEntry->hres);
   CString sError = comErr.ErrorMessage();
   hres = WriteIDS(IDS_TABLE_STRING, sError);
   ASSERT_AND_RETURN(hres);

   hres = SetRowEnd(sTableEnd);
   ASSERT_AND_RETURN(hres);
   return hres;
}

STDMETHODIMP CComLog::LogMethod(LOG_ENTRY *pMethodEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;
   CString sTableEnd;
   hres = NewRow(pMethodEntry, sTableEnd);
   ASSERT_AND_RETURN(hres);

   CString sStr;
   sStr = pMethodEntry->bstrMethodName;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   sStr = pMethodEntry->bstrSourceFileName;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   hres = WriteIDS(IDS_TABLE_DEC, pMethodEntry->dwLineNumber);
   ASSERT_AND_RETURN(hres);

   hres = SetRowEnd(sTableEnd);
   ASSERT_AND_RETURN(hres);

   return hres;
}

STDMETHODIMP CComLog::LogEvent(LOG_ENTRY *pEventEntry)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState())

   HRESULT hres = S_OK;
   CString sTableEnd;
   CString sStr;
   hres = NewRow(pEventEntry, sTableEnd);
   ASSERT_AND_RETURN(hres);

   sStr = pEventEntry->bstrDescription;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   hres = SetRowEnd(sTableEnd);
   ASSERT_AND_RETURN(hres);

   return hres;
}
/////////////////////////////////////////////////////////////
//Helper methods
void CComLog::GetTime(const LOG_ENTRY* pEvent, SYSTEMTIME& systemTime) const
{
   ::ZeroMemory(&systemTime, sizeof(systemTime));

   FILETIME localTime;
   BOOL bres = ::FileTimeToLocalFileTime(&pEvent->eventTime, &localTime);
   if (bres)
   {
      bres = ::FileTimeToSystemTime(&localTime, &systemTime);
   }
}

HRESULT CComLog::WriteString(const CString& sString)
{
   HRESULT hres = S_OK;

   if (m_LogFile.m_pStream)
   {
      TRY
      {
         m_LogFile.WriteString(sString);
         m_LogFile.Flush();   
      }
      CATCH(CFileException, e)
      {
         hres = E_FAIL;
      }
      END_CATCH
   }

   return hres;
}

HRESULT CComLog::WriteIDS(UINT uID)
{
   HRESULT hres = E_FAIL;
   CString sStr;
   BOOL bres = sStr.LoadString(uID);
   if (TRUE == bres)
   {
      hres = WriteString(sStr);
   }

   return hres;
}

HRESULT CComLog::WriteIDS(UINT uID, const CString& sString)
{
   CString sStr;
   sStr.Format(uID, sString);
   return WriteString(sStr);
}

HRESULT CComLog::WriteIDS(UINT uID, DWORD dwNumber)
{
   CString sStr;
   sStr.Format(uID, dwNumber);

   return WriteString(sStr);
}

HRESULT CComLog::WriteIDS(UINT uID, const SYSTEMTIME& systemTime)
{
   CString sStr;
   sStr.Format(uID,
               systemTime.wMonth, systemTime.wDay,    systemTime.wYear,
               systemTime.wHour,  systemTime.wMinute, systemTime.wSecond);

   return WriteString(sStr);
}

HRESULT CComLog::SetTableHeader(void)
{
   HRESULT hres;
   hres = WriteIDS(IDS_TABLE_HEADER);
   ASSERT_AND_RETURN(hres);

   hres = WriteIDS(IDS_TABLE_LINE_START);
   ASSERT_AND_RETURN(hres);

   CString sTableHeader;
   BOOL bres = sTableHeader.LoadString(IDS_HEADER_LABELS);
   if (FALSE == bres)
   {
      return E_FAIL;
   }

   int nIndex = sTableHeader.Find(',');
   while (nIndex >= 0)
   {
      CString sToken     = sTableHeader.Left(nIndex);
      CString sRemainder = sTableHeader.Mid(nIndex + 1);
      sTableHeader = sRemainder;

      hres = WriteIDS(IDS_TABLE_STRING, sToken);
      ASSERT_AND_RETURN(hres);

      nIndex = sTableHeader.Find(',');
   }

   hres = WriteIDS(IDS_TABLE_LINE_END);
   ASSERT_AND_RETURN(hres);

   WriteIDS(IDS_TABLE_END);

   return hres;
}

HRESULT CComLog::WriteGUID(const CString& sNULLGUID, REFGUID guid)
{
   CString sStr;

   if (GUID_NULL == guid)
   {
      sStr = sNULLGUID;
   }
   else
   {
      USES_CONVERSION;
      const int nSize = 64;
      WCHAR wszID[nSize];
      int nNumChars = ::StringFromGUID2(guid, wszID, nSize);
      VERIFY(nNumChars);
      sStr = W2A(wszID);
   }

   HRESULT hres = S_OK;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   return hres; 
}

HRESULT CComLog::NewRow(const LOG_ENTRY* pEvent, CString& sTableEnd)
{
   HRESULT hres = S_OK;
   BOOL bres = sTableEnd.LoadString(IDS_TABLE_END);
   if (FALSE == bres)
   {
      hres = E_FAIL;
      ASSERT_AND_RETURN(hres);
   }

   if (m_LogFile.m_pStream)
   {
      TRY
      {
         m_LogFile.Seek(-(sTableEnd.GetLength() + 1), CFile::current);
      }
      CATCH(CFileException, e)
      {
         hres = E_FAIL;
         ASSERT_AND_RETURN(hres);
      }
      END_CATCH
   }
   else
   {
      return E_FAIL;
   }

   hres = WriteIDS(IDS_TABLE_LINE_START);
   ASSERT_AND_RETURN(hres);

   SYSTEMTIME systemTime;
   GetTime(pEvent, systemTime);
   hres = WriteIDS(IDS_TABLE_TIME, systemTime);
   ASSERT_AND_RETURN(hres);

   CString sStr;
   sStr = pEvent->bstrMachineName;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   hres = WriteIDS(IDS_TABLE_DEC, pEvent->dwProcessID);
   ASSERT_AND_RETURN(hres);

   hres = WriteIDS(IDS_TABLE_HEX16, pEvent->dwThreadID);
   ASSERT_AND_RETURN(hres);

   hres = WriteGUID(CString("Default Context"), pEvent->guidContextID);
   ASSERT_AND_RETURN(hres);

   hres = WriteGUID(CString("No Transaction"), pEvent->guidTransactionID);
   ASSERT_AND_RETURN(hres);

   hres = WriteGUID(CString("No Activity"), pEvent->guidActivityID);
   ASSERT_AND_RETURN(hres);


   sStr = pEvent->bstrModuleName;
   hres = WriteIDS(IDS_TABLE_STRING, sStr);
   ASSERT_AND_RETURN(hres);

   return hres;
}

HRESULT CComLog::SetRowEnd(const CString& sTableEnd)
{
   HRESULT hres = S_OK;
   hres = WriteIDS(IDS_TABLE_LINE_END);
   ASSERT_AND_RETURN(hres);

   hres = WriteString(sTableEnd);
   ASSERT_AND_RETURN(hres);

   return hres;
}


